class Button 
{
  PGraphics layer;
  float x, y; // button-centre coords
  String BText1, BText2;       // button text
  int BStateFlag;       // button state: 0 = free / 1 = selected / 2 = pressed (selected instantly)
  boolean WasBSelected;  // previous button state: false = free // true = selected
  
  float BWidth, BHeight; // button dimensions
  
  int fontsize; // calculate fontsize based on text string and internal button size
  
  float c = 2, b = 4; // arbitrary constants (b used to be 4)
  float SAlpha;

  color ContentColour, BColour;
  color shade, light, plateau, selected;
  
  float dxoffset, dyoffset; // offsets of window (layer) where button is being placed at relative to global (0,0) (TOP LEFT CORNER)
  
  Button(PGraphics _layer, float _dxoffset, float _dyoffset, float _x, float _y, 
         float _BWidth, float _BHeight, String _BText1, String _BText2, color _BColour) {
    x = _x;
    y = _y;
    BWidth = _BWidth;
    BHeight = _BHeight;
    BStateFlag = 0;      // Button is initialised as free, default behaviour.
    WasBSelected = false;  // If needed, change var after constructor function call
    BText1 = _BText1;
    BText2 = _BText2;
    layer = _layer;
    dxoffset = _dxoffset;
    dyoffset = _dyoffset;
    BColour = _BColour;
    
    SAlpha = 255;
    SetColours();
  }
  
  
  void SetColours() {
    plateau = BColour;
    shade = lerpColor(#000000, BColour, 0.8); shade = color(red(shade), green(shade), blue(shade), alpha(BColour));
    light = lerpColor(#FFFFFF, BColour, 0.8); light = color(red(light), green(light), blue(light), alpha(BColour));
    selected = color(255-red(BColour), 255-green(BColour), 255-blue(BColour));
  }
  
void DisplayColours() {
    color TMPcolour;
    
    TMPcolour = plateau;
    
    layer.fill(TMPcolour);
    layer.noStroke();
    layer.rectMode(CENTER);
    layer.rect(x, y, BWidth, BHeight);
    
    switch(BStateFlag) {
      case 0: // free
        layer.fill(light);
        break;
      case 1: // selected
        layer.fill(shade);
        break;
      case 2: // pressed
        layer.fill(lerpColor(selected, shade, 0.9));
        break;
    }
    layer.beginShape(TRIANGLE_STRIP); // split in 2 shapes to avoid WEIRD blending effect after color change
    layer.vertex(x + BWidth/2 - b, y - BHeight/2 + b);
    layer.vertex(x + BWidth/2, y - BHeight/2);
    layer.vertex(x - BWidth/2 + b, y - BHeight/2 + b);
    layer.vertex(x - BWidth/2, y - BHeight/2);
    layer.vertex(x - BWidth/2 + b, y + BHeight/2 - b);
    layer.vertex(x - BWidth/2, y + BHeight/2);
    layer.endShape();
    
    switch(BStateFlag) {
      case 0: // free
        layer.fill(shade);
        break;
      case 1: // selected
        layer.fill(light);
        break;
      case 2: // pressed
        layer.fill(lerpColor(selected, light, 0.9));
        break;
    }
    layer.beginShape(TRIANGLE_STRIP);
    layer.vertex(x - BWidth/2 + b, y + BHeight/2 - b);
    layer.vertex(x - BWidth/2, y + BHeight/2);
    layer.vertex(x + BWidth/2 - b, y + BHeight/2 - b);
    layer.vertex(x + BWidth/2, y + BHeight/2);
    layer.vertex(x + BWidth/2 - b, y - BHeight/2 + b);
    layer.vertex(x + BWidth/2, y - BHeight/2);
    layer.endShape();
    
    
    switch(BStateFlag) { // this is the colour of the text, left ready to be applied in text created in display, after this funciton call
      case 0: // free
        ContentColour = lerpColor( color((brightness(TMPcolour) > 128) ? (0) : (255)) , shade, 0.2);
        break;
      case 1: // selected
        ContentColour = lerpColor( color((brightness(TMPcolour) > 128) ? (0) : (255)) , light, 0.2);
        break;
      case 2: // pressed
        ContentColour = lerpColor( color((brightness(TMPcolour) > 128) ? (0) : (255)) , TMPcolour, 0.7);
        break;
    }
  }
  
  void display() {
    layer.pushStyle();
    DisplayColours();
    layer.fill(ContentColour);
    layer.textAlign(CENTER, CENTER);
    layer.textFont (helv, 16);
    if (isSelected())
    {
      layer.fill (255);
      layer.text(BText2, x, y);
    }
    else
      layer.text(BText1, x, y);
    layer.popStyle();
  }
  
  void mousePressed(float mX, float mY) {
    float dx = abs(mX - (x + dxoffset));
    float dy = abs(mY - (y + dyoffset));
    //dx e dy: testar posicao do mouse pra ver se quando eh clicado estah sobre o botao
    //mouseButton: testar pra soh considerar clique com botao esquerdo (botao direito eh inocuo)
    //se clicou com o esquerdo E estah dentro da area do botao
    if (mouseButton == LEFT && dx <= BWidth/2 && dy <= BHeight/2) {
      switch(BStateFlag) {
        case 0: // se estah FREE
          BStateFlag = 2; // vai pra PRESSED
          WasBSelected = false; // e antes estava FREE
          break;
        case 1: // se estah SELECTED
          BStateFlag = 2; // vai pra PRESSED
          WasBSelected = true; // e antes estava SELECTED
          break;
      }
    }
  }
  
  void mouseReleased() {
    //se estah PRESSED e estava FREE -> SELECTED
    //se estah PRESSED e estava SELECTED -> FREE
    if(BStateFlag == 2) {
      BStateFlag = (WasBSelected) ? (0) : (1);
    }
  }
  
  void free() {
    WasBSelected = true;
    BStateFlag = 0;
  }
  
  boolean isSelected() {
    if (BStateFlag == 1) return true;
    else return false;
  }
  
  boolean wasSelected() {
    return WasBSelected;
  }
  
  boolean isPressed() {
    if (BStateFlag == 2) return true;
    else return false;
  }
  
  void Select() {
    BStateFlag = 1;
    WasBSelected = false;
  }
  
  void unSelect () {
    BStateFlag = 0;
    WasBSelected = true;
  }
    
  
  void Press() {
    BStateFlag = 2;
  }
  
  float GetParameters(char p) {
    float TMP = 0;
    switch(p) {
      case 'x': TMP = x; break;
      case 'y': TMP = y; break;
      case 'w': TMP = BWidth; break;
      case 'h': TMP = BHeight; break;
    }
    return TMP;
  }
  
  String GetText() {
    if (isSelected())
    return BText2;
    else
    return BText2;
  }
  
  color GetBGColour() {
    return BColour;
  }
  
  void SetPosition(float _x, float _y) {
    x = _x; y = _y;
  }
  
  void SetSize(float _BWidth, float _BHeight) {
    BWidth = _BWidth; BHeight = _BHeight;
  }
  
  void SetText(String _BText1, String _BText2) {
    BText1 = _BText1;
    BText2 = _BText2;
  }
  
  void SetBGColour(color _BColour) {
    BColour = _BColour;
    //Update Stuff
    //SetColours();
  }
  
  void SetWindow(PGraphics _layer, float _dxoffset, float _dyoffset) {
    dxoffset = _dxoffset; dyoffset = _dyoffset;
    if (_layer != null) layer = _layer; // flag, to only set offsets
  }
}

