void buildScenario (PGraphics layer)
{
  float areaOffset=5;

  layer.background (SimBG);
  layer.fill(0);

  layer.beginDraw();

  layer.pushMatrix();
  layer.pushStyle();
  layer.translate(Ox, Oy);

  layer.noFill();
  layer.stroke(255);
  layer.strokeWeight(2);

  layer.beginShape();                                              // circuit
  layer.vertex(Hsize/2-Harea/2, Varea/2);
  layer.vertex(radius, Varea/2);
  layer.bezierVertex(0, Varea/2, 0, Varea/2, 0, Varea/2+radius);
  layer.vertex(0, Vsize-radius);
  layer.bezierVertex(0, Vsize, 0, Vsize, radius, Vsize);
  layer.vertex (Hsize-radius, Vsize);
  layer.bezierVertex(Hsize, Vsize, Hsize, Vsize, Hsize, Vsize-radius);
  layer.vertex (Hsize, Varea/2+radius);
  layer.bezierVertex (Hsize, Varea/2, Hsize, Varea/2, Hsize-radius, Varea/2);
  layer.vertex(Hsize/2+Harea/2, Varea/2);
  layer.endShape();


  layer.fill (160,220,255);
  layer.rect (Hsize/2-Harea/2, 0, Harea, Varea);                  // test area
  layer.fill (SimBG);
  layer.rect (Hsize/2-Harea/2+areaOffset, +areaOffset, Harea-2*areaOffset, Varea-2*areaOffset);



  // generator
  layer.pushStyle();
  layer.fill(SimBG);
  layer.rect(Hsize/4, Vsize-femDim/2, 4*femDim, femDim);
  layer.popStyle();
  //layer.line(0.25*Hsize, Vsize-0.7*femDim, 0.25*Hsize, Vsize+0.7*femDim);
  //layer.line(0.25*Hsize+0.16*femDim, Vsize-femDim, 0.25*Hsize+0.16*femDim, Vsize+femDim);
  //layer.line(0.25*Hsize+0.5*femDim, Vsize-1.2*femDim, 0.25*Hsize+0.5*femDim, Vsize-0.8*femDim);
  //layer.line(0.25*Hsize+0.3*femDim, Vsize-femDim, 0.25*Hsize+0.7*femDim, Vsize-femDim);

  layer.line(Hsize/2-Harea/2, Varea/2, Hsize/2, Varea/2);                // cathode wire
  layer.line(Hsize/2+Harea/2, Varea/2, Hsize/2, Varea/2);                // anode wire


  layer.popStyle();
  layer.popMatrix();
  layer.endDraw();
}

void buildCathode (PGraphics layer, color colour)
{  
  layer.pushMatrix();
  layer.translate (Ox, Oy);
  layer.translate (Hsize/2, Varea/2);
  layer.pushStyle();
  layer.fill(150);                                            // change the parameter to "colour" to show the current selected color on the emitter
  layer.arc (0,0, 1.1*catRad, 1.1*catRad, PI-ang, PI+ang);
  layer.fill(SimBG);
  layer.arc (0,0, 0.9*catRad, 0.9*catRad, PI-ang, PI+ang);
  layer.pushMatrix();
  layer.rotate (-0.01);
  layer.line(catRad/2*1.1*cos(PI-ang), catRad/2*1.1*sin(PI-ang), catRad/2*0.9*cos(PI-ang), catRad/2*0.9*sin(PI-ang));
  layer.line(catRad/2*1.1*cos(PI+ang), catRad/2*1.1*sin(PI+ang), catRad/2*0.9*cos(PI+ang), catRad/2*0.9*sin(PI+ang));
  layer.fill (255);
  layer.strokeWeight(2);
  layer.ellipse(0, 0, 10, 10);                                                  // anode
  layer.popMatrix();
  
  layer.fill (colour);
  layer.stroke(150);
  layer.strokeWeight(2);
  layer.ellipse (-175, 130, 50, 50); 
  
  layer.popStyle();
  layer.popMatrix();
}

void buildAmp (PGraphics layer)
{
  float I;
  layer.pushMatrix();
  layer.pushStyle();
  layer.noStroke();
  layer.translate (Ox, Oy);
  layer.translate (Hsize/2, Varea/2);
  layer.translate (Harea*0.62,0);
  layer.rectMode(CENTER);
  layer.fill(0,100,255);
  layer.rect (0,-40,80,120);
  layer.fill(255);
  layer.rect (0,-55,70,80);
  layer.fill(0);
  layer.translate(0,-25);
  layer.ellipse (0,0,4,4);
  layer.noFill();
  layer.stroke(0);
  layer.arc (0,0,80,80,5*PI/4, 7*PI/4);


  I=bezierPoint (bezTop, bezTop, bezBottom+bezTan, bezBottom, t);        // amperimeter indication follows graphic Y-value
  if (I>0) I=0;
  I=map(I, 36, -108, 5*PI/4, 3*PI/4);                                    // 36=min bezBottom and -108=max bezBottom   NOTE: see opposite values *(-1)
  if (sourceWL<catWL) layer.line (0,0, 48*sin(I), 48*cos(I));            // draw indicator if Photoeletric Effect is going on
  else                layer.line (0,0, 48*sin(9*PI/8), 48*cos(9*PI/8));  // zero current if Photoeletric Effect isn't happening


  layer.line (40*sin(5*PI/4), 40*cos(5*PI/4), 48*sin(5*PI/4), 48*cos(5*PI/4));    // minimum mark
  layer.line (40*sin(3*PI/4), 40*cos(3*PI/4), 48*sin(3*PI/4), 48*cos(3*PI/4));    // maximum mark
  layer.stroke (0,0,255);
  layer.fill (0,0,255);
  layer.line (40*sin(9*PI/8), 40*cos(9*PI/8), 48*sin(9*PI/8), 48*cos(9*PI/8));    // ZERO mark
  layer.textFont (helv, 8);
  layer.text ("0",  55*sin(9*PI/8), 55*cos(9*PI/8));                              // ZERO indication
  layer.textFont (verdanab, 10);
  layer.fill (255,0,0);
  layer.text ("+",  -5+30*sin(3*PI/4), 30*cos(3*PI/4));                           // Positive side
  layer.fill (0);
  layer.text ("-",  30*sin(5*PI/4), 30*cos(5*PI/4));                              // Negative side

  layer.textFont (helv, 13);
  layer.fill(255);
  layer.text ("Amperímetro",-36,30);



  layer.popStyle();
  layer.popMatrix();
}

class FEMindicator
{
  PGraphics layer;
  float anchorX, anchorY, x, y, index;
  color[] leftGrad, rightGrad;
  color left, center, right;
  int IWidth, IHeight, pc;

  FEMindicator (PGraphics _layer, float _anchorX, float _anchorY, float _x, float _y, color left, color center, color right, int _IWidth, int _IHeight)
  {
    layer=_layer;
    IWidth =_IWidth;
    IHeight=_IHeight;
    anchorX =_anchorX;
    anchorY =_anchorY;
    x=_x;
    y=_y;

    leftGrad  = new color[IWidth/2];
    rightGrad = new color[IWidth/2];
    setColors (left, center, right);
  }

  void setColors (color left, color center, color right)
  {
    for (int i=0; i<IWidth/2; i++)
    {
      index = float(i+30)/float(IWidth/2);
      index = constrain (index, 0, 1);
      leftGrad[i]  = lerpColor (center, left, index);
      rightGrad[i] = lerpColor (center, right, index);
      //println(index);
    }
  }

  void display (float percentage)
  {
    layer.pushMatrix();
    layer.pushStyle();

    percentage = constrain (percentage, -100, 100);
    layer.translate (anchorX+x+IWidth/2, anchorY+y);
    layer.fill(255);
    layer.textFont (verdanab, 20);

    if (percentage==0) 
    {
      layer.stroke(255);
      layer.line(0,0,0,IHeight);
    }
    if (percentage>0)
    {
      for(int i=0; i<(percentage/100*(IWidth/2)); i++)
      {
        layer.stroke (rightGrad[i]);
        layer.line(i,0,i,IHeight);
      }
      
      layer.text ("+",IWidth/2-22,IHeight/2+8);
    }
    if (percentage<0)
    {
      for(int i=0; i<(abs(percentage)/100*(IWidth/2)); i++)
      {
        layer.stroke (leftGrad[i]);
        layer.line(-i,0,-i,IHeight);
      }
      layer.text ("+",-IWidth/2+6, IHeight/2+8);
    }
    layer.popMatrix();
    layer.popStyle();
  }
}




